@@ -1,4 +1,4 @@
-#!/usr/pkg/bin/python3.7
+#!/usr/pkg/bin/python3.8
 
 """Python interface for the 'lsprof' profiler.
    Compatible with the 'profile' module.
@@ -112,6 +112,9 @@ class Profile(_lsprof.Profiler):
         elif 'func' in kw:
             func = kw.pop('func')
             self, *args = args
+            import warnings
+            warnings.warn("Passing 'func' as keyword argument is deprecated",
+                          DeprecationWarning, stacklevel=2)
         else:
             raise TypeError('runcall expected at least 1 positional argument, '
                             'got %d' % (len(args)-1))
@@ -121,6 +124,14 @@ class Profile(_lsprof.Profiler):
             return func(*args, **kw)
         finally:
             self.disable()
+    runcall.__text_signature__ = '($self, func, /, *args, **kw)'
+
+    def __enter__(self):
+        self.enable()
+        return self
+
+    def __exit__(self, *exc_info):
+        self.disable()
 
 # ____________________________________________________________
 
@@ -157,6 +168,11 @@ def main():
     (options, args) = parser.parse_args()
     sys.argv[:] = args
 
+    # The script that we're profiling may chdir, so capture the absolute path
+    # to the output file at startup.
+    if options.outfile is not None:
+        options.outfile = os.path.abspath(options.outfile)
+
     if len(args) > 0:
         if options.module:
             code = "run_module(modname, run_name='__main__')"
@@ -175,7 +191,12 @@ def main():
                 '__package__': None,
                 '__cached__': None,
             }
-        runctx(code, globs, None, options.outfile, options.sort)
+        try:
+            runctx(code, globs, None, options.outfile, options.sort)
+        except BrokenPipeError as exc:
+            # Prevent "Exception ignored" during interpreter shutdown.
+            sys.stdout = None
+            sys.exit(exc.errno)
     else:
         parser.print_usage()
     return parser
